# -*- coding: utf-8 -*-
'''****************************************************************************
***  This file is part of HgWeb Manager.                                    ***
***                                                                         ***
***  Copyright (C) 2014 - 2016                                              ***
***  CodeGNU Solutions <Licensing _AT_ CodeGNU _DOT_ com>                   ***
***                                                                         ***
***  This program is free software: you can redistribute it and/or modify   ***
***  it under the terms of the GNU Affero General Public License as         ***
***  published by the Free Software Foundation, either version 3 of the     ***
***  License, or (at your option) any later version.                        ***
***                                                                         ***
***  This program is distributed in the hope that it will be useful, but    ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of             ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                   ***
***  See the GNU Affero General Public License for more details.            ***
***                                                                         ***
***  You should have received a copy of the GNU Affero General Public       ***
***  License along with this program. If not,                               ***
***  see <http://www.gnu.org/licenses/>.                                    ***
****************************************************************************'''

import enum


# The types of filtering available for an ignore file.
FilterType = enum.create('Glob', 'RegularExpression')


def ignore_syntax_name(mapFilterType):
    """ Provides the mapping between an enumerated filter type and the
    Mercurial ignore section header.

    @param[in] mapFilterType    Is the enumerated value to map.

    @throws ValueError    When an invalid filtering type is supplied.

    @return The string value of the supplied enumerated value.
    """
    if FilterType.Glob == mapFilterType:
        return 'glob'
    elif FilterType.RegularExpression == mapFilterType:
        return 'regexp'
    else:
        raise ValueError("Unknown filter type supplied, '%s'." % mapFilterType)


class Pattern(object):
    """ Describes the properties of an entry in a Mercurial ignore file. """

    # A short entry describing the purpose of the ignore entry.
    __description = None
    # The type of ignore entry used by Mercurial.
    __filter = None
    # The ignore expression to provide for an ignore file.
    __value = None

    @property
    def Description(self):
        """ A short entry describing the purpose of the ignore entry when
        provided, else gives miscellaneous.
        """
        if self.__description:
            return self.__description
        else:
            return 'Miscellaneous'

    @property
    def Filter(self):
        """ The type of ignore entry used by Mercurial. """
        return self.__filter

    @property
    def Value(self):
        """ The ignore expression to provide for an ignore file. """
        return self.__value

    def __init__(self, desc, fil, val):
        """ Initialises the object with the values supplied.

        @param[in] desc    The description of the ignore entry.
        @param[in] fil    The type of ignore filtering.
        @param[in] val    The ignore expression.

        @throws ValueError        When the supplied ignore filter is invalid.
        """
        self.__description = desc.strip()
        if fil == 'glob':
            self.__filter = FilterType.Glob
        elif fil == 'regex':
            self.__filter = FilterType.RegularExpression
        else:
            raise ValueError("Invalid filter type '%s' supplied" % fil)
        self.__value = val.strip()


def extract(ignoreFile):
    """ Parses an XML file for ignore file patterns. Generates a dictionary
    object whose key is the name of the group for a collection of ignore
    patterns. The value in each dictionary is an array of all the ignore
    pattern objects within that group. Upon any parsing error, no ignore
    patterns are supplied.

    @param[in] ignoreFile    The directory path and file name to an XML file
                             that contains the ignore patterns to extract.

    @throw ValueError       When the name of a group is reused in the XML file.
    @throw RuntimeError     When the XML file cannot be parsed.

    @return The extracted grouped ignore patterns.
    """
    from xml.dom import minidom
    from xml.parsers.expat import ExpatError

    ignores = {}
    try:
        xmlIgnore = minidom.parse(ignoreFile)
    except ExpatError as error:
        raise RuntimeError(error)
    for group in xmlIgnore.getElementsByTagName(
            'ignores')[0].getElementsByTagName('group'):
        groupName = group.getAttribute('name')
        if groupName in ignores:
            raise ValueError("The group '%s' was already defined." % groupName)
        else:
            ignores[groupName] = []
        for pattern in group.getElementsByTagName('pattern'):
            ignores[groupName].append(Pattern(
                pattern.getAttribute('description'),
                pattern.getAttribute('type'),
                pattern.firstChild.nodeValue))
    return ignores
