# -*- coding: utf-8 -*-
'''****************************************************************************
***  This file is part of HgWeb Manager.                                    ***
***                                                                         ***
***  Copyright (C) 2014 - 2016                                              ***
***  CodeGNU Solutions <Licensing _AT_ CodeGNU _DOT_ com>                   ***
***                                                                         ***
***  This program is free software: you can redistribute it and/or modify   ***
***  it under the terms of the GNU Affero General Public License as         ***
***  published by the Free Software Foundation, either version 3 of the     ***
***  License, or (at your option) any later version.                        ***
***                                                                         ***
***  This program is distributed in the hope that it will be useful, but    ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of             ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                   ***
***  See the GNU Affero General Public License for more details.            ***
***                                                                         ***
***  You should have received a copy of the GNU Affero General Public       ***
***  License along with this program. If not,                               ***
***  see <http://www.gnu.org/licenses/>.                                    ***
****************************************************************************'''

import errno
import os
import tempfile
import time


repositoryLockFile = os.path.join(tempfile.gettempdir(), 'HWM.lock')


class RepositoryManagerLock(object):
    """ Handles the locking for the repository manager. """

    def __init__(self):
        """ Initialises the lock for use, but does not get one. """
        self.haveLock = False

    def __isLockedProcessRunning(self):
        """ Determines if a process is already running

        @return When the process is running gives true, false else-wise.
        """
        try:
            with open(repositoryLockFile) as previousLock:
                pid = int(previousLock.readline())
            os.kill(pid, 0)
            return True
        except OSError:
            return False  # Process is not running.

    def acquire(self):
        """ Gets the repository lock. """
        while True:
            try:
                print 'Acquiring repository manager lock ...'
                self.lock = os.open(repositoryLockFile, os.O_CREAT |
                                    os.O_EXCL | os.O_RDWR)
                break
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise

                if not self.__isLockedProcessRunning():
                    print 'A dead repository lock found, removing ...'
                    os.unlink(repositoryLockFile)
                time.sleep(0.75)
        self.haveLock = True
        os.write(self.lock, str(os.getpid()))

    def release(self):
        """ Frees the repository lock. """
        if self.haveLock:
            print 'Releasing repository manager lock ...'
            os.close(self.lock)
            os.unlink(repositoryLockFile)
            self.haveLock = False

    def __enter__(self):
        if not self.haveLock:
            self.acquire()
        return self

    def __exit__(self, type, value, traceback):
        if self.haveLock:
            self.release()

    def __del__(self):
        """ Ensures that when the object is gone, the lock is released. """
        self.release()
